package com.scurab.dictgen;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.scurab.dictgen.Keyboard.Key;
import com.scurab.dictgen.Keyboard.Row;
import com.scurab.dictgen.Test.StructCorrect;

public class DBGenerator {

	private final String insertTemplate = "INSERT OR IGNORE INTO \"%s\" (\"wordbase\",\"normwordbase\",\"searchtext\",\"gensearchtext\", \"swype\", \"genswype\", \"isbwu\",\"suffix\",\"cntr\", \"wrdlen\") " +
											"VALUES   (\"%s\"      ,\"%s\"          ,\"%s\"        ,\"%s\"           , \"%s\"   , \"%s\"      , %s       , %s       ,   0    ,%s);";
	private final String insertCompleteTemplate = "INSERT INTO \"%s\" (\"wordbase\",\"normwordbase\",\"genwordbase\",\"searchtext\",\"gensearchtext\", \"swype\", \"genswype\" ,\"isbwu\",\"suffix\",\"cntr\", \"wrdlen\") " +
			"											VALUES(\"%s\"      ,\"%s\"          ,\"%s\"         ,\"%s\"        ,\"%s\"           , \"%s\"   , \"%s\"       ,%s        ,%s       ,  0     ,%s);";
	
	private int maxRecordsInTable = 5000;
//	private String filesLocation = null;
//	private String dbFile = null;
	private int aggreggLimit = 50;
	private int lenLimit = 99;
	private Keyboard keyboard_qwerty = new Keyboard();
	private Keyboard keyboard_qwertz = new Keyboard();
	
	
	private HashMap<String, String > tableMap = new HashMap<String, String>();
	private HashMap<Integer, String > suffixy = new HashMap<Integer, String>();
	private HashMap<Integer, String > normSuffixy = new HashMap<Integer, String>();
	
	Connection conn = null;
    Statement stat = null;
    Params params = null;
    
//	public DBGenerator(String dbFile) {
//		this.dbFile = dbFile;
//		//this.fillTables();
//	}
    
    public DBGenerator(Params params) throws ClassNotFoundException, SQLException {
    	this.params = params;
		String xmlKeyboard = "qwerty.xml";
		keyboard_qwerty = this.loadKeyboardXML(xmlKeyboard);
		xmlKeyboard = "qwertz.xml";
		keyboard_qwertz = this.loadKeyboardXML(xmlKeyboard);
		this.initDbFile();
    }
//	}
    
    public void close() throws SQLException
    {
    	stat.close();
    	conn.close();
    }
	
	public void createEmptyUserDatabase() throws ClassNotFoundException, SQLException, NumberFormatException, IOException
	{
		this.createAndroidMetaData();
		
		 String SQL_CREATE_DICTIONARY_TABLE = "CREATE VIRTUAL TABLE  [%s] USING fts3(\"wordbase\" TEXT PRIMARY KEY NOT NULL UNIQUE, \"normwordbase\" TEXT NOT NULL, \"searchtext\" TEXT NOT NULL, \"gensearchtext\" TEXT NOT NULL,  \"swype\" TEXT NOT NULL, \"genswype\" TEXT NOT NULL, \"isbwu\" INTEGER NOT NULL, \"suffix\" INTEGER NOT NULL, \"cntr\" INTEGER INTEGER DEFAULT 0, \"wrdlen\" INTEGER NOT NULL, \"profil\" INTEGER );";
//	 	 String SQL_CREATE_COUNTER_TABLE = "CREATE VIRTUAL TABLE  [cntrs_%s] USING fts3(\"id\" INTEGER NOT NULL, \"tablename\" TEXT NOT NULL UNIQUE, \"user\" INTEGER NOT NULL, \"cntr\" INTEGER NOT NULL);";
		 String SQL_CREATE_COUNTER_TABLE = "CREATE VIRTUAL TABLE  [cntrs_%s] USING fts3(\"wordbase\" TEXT NOT NULL UNIQUE, \"user\" INTEGER NOT NULL, \"cntr\" INTEGER NOT NULL);";
	 	 String SQL_CREATE_SUFFIX_TABLE = "CREATE VIRTUAL TABLE \"suff\" USING fts3(\"id\" INTEGER PRIMARY KEY UNIQUE NOT NULL, \"suffix\" TEXT NOT NULL, \"normsuffix\" TEXT NOT NULL);";
	 	 
		char[] alphabet = "abcdefghijklmnopqrstuvwxyz".toUpperCase().toCharArray();
		String sql = null;
		for(char c : alphabet)
		{
			sql = String.format(SQL_CREATE_DICTIONARY_TABLE,c);
			this.stat.execute(sql);
			sql = String.format(SQL_CREATE_COUNTER_TABLE,c);
//			this.stat.execute(sql);
		}
		sql = SQL_CREATE_SUFFIX_TABLE;
		this.stat.execute(sql);				
	}
	
	public void startGenUserDB() throws Exception
	{		
//		this.filesLocation = filesLocation + "%s.txt";
						
		if(!params.appendToUserDB)
		{
			this.createEmptyUserDatabase();
		}
		
		this.parseUserFile(getSortedUserFile());
		this.echoRecordCount();
		//this.moveRecordsFromSmallTables();
		System.out.print("Done...");
	}
	
	private ArrayList<String> getSortedUserFile() throws IOException
	{
		ArrayList<String> list = new ArrayList<String>();
		BufferedReader br;
		//Reader in = new InputStreamReader(new FileInputStream(params.userSourceFile),"UTF-8");
		Reader in = new InputStreamReader(new FileInputStream(params.userSourceFile));
		br = new BufferedReader(in);
		String line = null;
		while((line = br.readLine()) != null)
		{
			list.add(line);
		}
		Collections.sort(list);
		return list;
	}
	
	private void parseUserFile(List<String> rows) throws NumberFormatException, IOException, SQLException
	{
		int suffCntr = 1;
		if(params.appendToUserDB)
		{
			
		}
		
		HashMap<String,Integer> tempSuffixes = new HashMap<String,Integer>();
		for(String line : rows)
		{
			String suffix = getSuffixesFromUserLine(line);
			int idSuffix = 0;
			if(tempSuffixes.containsKey(suffix))
				idSuffix = tempSuffixes.get(suffix);
			else
			{
				idSuffix = insertUserSuffix(suffix);
				tempSuffixes.put(suffix, idSuffix);
			}
			String[] arr = line.split(" ");
			
			String word = arr[0].toLowerCase();
			String usbw = arr[1];
			insertUserWord(word,usbw, idSuffix);
		}
	}
	
	private void insertUserWord(String baseword, String usbw, int suffixID) throws SQLException
	{
				
		String normWord = Stringology.getNormalizedString(baseword);
		int angleLimit = 55;
		int angleLimit2 = 90;
		int angleLimit3 = 120;
		String ssy = this.getSwypeText(normWord,angleLimit,this.keyboard_qwerty);
		String ssyl = this.getSwypeText(normWord,angleLimit3,this.keyboard_qwerty);
		String ssyl2 = this.getSwypeText(normWord,angleLimit2,this.keyboard_qwerty);
		String ssz = this.getSwypeText(normWord,angleLimit,this.keyboard_qwertz);
		String sszl = this.getSwypeText(normWord,angleLimit3,this.keyboard_qwertz);
		String sszl2 = this.getSwypeText(normWord,angleLimit2,this.keyboard_qwertz);
		String swypeValue = this.getDistinctWords(String.format("%s %s %s %s %s %s",ssy,ssyl,ssz, sszl,ssyl2,sszl2));
		//String swypeValue = this.getDistinctWords(String.format("%s %s %s %s",ssy,ssz, ssyl2,sszl2));
//					if(swypeValue.contains(" "))
//						swypeValue = swypeValue.replace(normWord, "").replace("  ", " ").trim();
//					swypeValue = getShortestWord(swypeValue);
//					if(swypeValue.contains(" "))
//					{
// 						swypeValue = ""; 
//					}
		String genSwypeValue = Stringology.getGeneralizedString(swypeValue);
		String table = normWord.substring(0,1).toUpperCase();
		
		String sql = null;
		PreparedStatement ps = null;			
		int basewordlen = baseword.length();
		String suff = this.normSuffixy.get(suffixID);
		String searchText = getSpaceSeparatedWord(normWord);				
		if(!(suff == null && suffixID == 0))
			searchText += " "  + this.getSearchableSuffix(suff);
		
		String genNormWord = Stringology.getMoreGeneralizedString(normWord);			
		String genSearchText = Stringology.getGeneralizedString(searchText);				
		sql = String.format(insertTemplate,table,baseword,normWord, searchText, genSearchText, swypeValue,genSwypeValue,usbw,suffixID,basewordlen);
		ps = this.conn.prepareStatement(sql);
		ps.execute();
	}
	
	private int insertUserSuffix(String suffix)
	{
		int suffId = -1;
		ResultSet rs = null;
		try
		{
			rs = this.stat.executeQuery(String.format("SELECT id FROM suff where suffix = '%s'",suffix));
			if(rs.next())
			{
				suffId = rs.getInt("id");
				this.suffixy.put(suffId, suffix);
				this.normSuffixy.put(suffId, Stringology.getNormalizedString(suffix));
			}
			rs.close();
			
			if(suffId == -1)
			{
				rs = this.stat.executeQuery("SELECT MAX(id) as maxid FROM suff");
				if(rs.next())
				{
					suffId = rs.getInt("maxid") + 1;
				}
				rs.close();
				
				final String insertTemplate = "INSERT INTO \"suff\" (\"id\",\"suffix\", \"normsuffix\") VALUES (%s,\"%s\",\"%s\");";
				String normSuffix = Stringology.getNormalizedString(suffix);						
				String sql = String.format(insertTemplate,suffId,suffix,normSuffix);
				PreparedStatement ps = this.conn.prepareStatement(sql);
				ps.execute();
				this.suffixy.put(suffId, suffix);
				this.normSuffixy.put(suffId, normSuffix);
			}
		}
		catch(Exception e)
		{
			suffId = 0;
		}
		return suffId;
	}
	
	private String getSuffixesFromUserLine(String line)
	{
		String[] atoms = line.split(" ");
		if(atoms.length < 3)
			return null;
		
		
		ArrayList<String> suffy = new ArrayList<String>();
		for(int i = 2;i<atoms.length;i++)
		{
			suffy.add(atoms[i]);
		}
		Collections.sort(suffy);
		StringBuilder sb = new StringBuilder();
		for(String suff: suffy)
				sb.append(suff + " ");		
		String suffix = sb.substring(0,sb.length() - 1);
		suffix = Stringology.getDistinctWords(suffix);
		return suffix;
	}
	
	
	public void startGenDefaultDB() throws Exception
	{		
		//android meta tabulka, musi tam bejt, jinak se mu moc nehce
		this.createAndroidMetaData();
		
		//map tabulka, kde jsou vsechny tabulky co reprezentuji datovy kontejnery
		this.createMapTable();
		//tabulka suffixu
		this.createSuffixTable();
		
		//naplni tabulku suffixu
		this.fillSuffixTable();
		
		//naparsuje 2 fajly a vlozi je do COMPLETE tabulky, tu pak rozdeluje z ohledem na limit 
		this.parseFile();

		//nic zajimavyho
		this.echoRecordCount();
		
		//zkontroluje slova, ktery maji kontext, a uzel ma jeste dalsi potomky
		//takovy slovo se expanduje
		//napr. neci 1 m ch. je ulozeno v Necu, ale jeste je dalsi patro stromu Necum...
		this.checkForShortWordsInLongWay();
		
		//tohle zkopiruje kratky slova jeste do kontejneru pro rodice.
		//Treba slovo než je uloženo v Ne. TOto je čistě kvůli optimalizaci
		this.moveShortBaseWordsToParentNode();
		
		//do map tabulky naleje jeste pocty zaznamu, v podstate jenom pro info
		this.updateMapTable();		
		
		System.out.print("Done...");
	}
	
	
	/**
	 * Tohle byla metoda, ktera prosla slova, ktery mely zaklady slov kratsi nez cesta ve stromu kontejneru
	 * @throws SQLException
	 */
	private void checkForShortWordsInLongWay() throws SQLException
	{
		String sql = "Select tablename FROM map ORDER BY prefix";
		ResultSet rs = this.stat.executeQuery(sql);
		ArrayList<String> tabs = new ArrayList<String>();				
		while(rs.next())			
		{
			String table = rs.getString("tablename");	
			tabs.add(table);
		}
		
		ArrayList<StructCorrect> toRepair = new ArrayList<StructCorrect>();
		
		for(String table : tabs)
		{
			String sqlTpl = String.format("Select * FROM [%s] WHERE wrdlen <= %s AND suffix > 0",table,table.length());
			rs = this.stat.executeQuery(sqlTpl);
			while(rs.next())
			{
				String word = rs.getString("wordbase");
				int idSuffix = rs.getInt("suffix");
				boolean isbwu = rs.getInt("isbwu") == 1;
				boolean isOk = isCorrectLength(tabs,word);
				StructCorrect sc = new StructCorrect();
				
				sc.table = table;
				sc.word = word;
				sc.idSuffix = idSuffix;
				sc.isbwu = isbwu;
				
				if(!isOk)
				{
					toRepair.add(sc);
//					System.out.printf("table:%s word:%s isOk:%s\n",table,word,isOk);
				}
			}
			rs.close();
		}
		
//		toRepair.clear(); 
//		StructCorrect sca = new StructCorrect();
//		sca.table = "Necu";
//		sca.word = "něčí";
//		sca.idSuffix = 27;
//		toRepair.add(sca);
		for(StructCorrect sc : toRepair)
		{
			sql = String.format("DELETE FROM [%s] WHERE wordbase='%s'",sc.table,sc.word);
			this.stat.execute(sql);
			handleShortWordsWithSuffis(sc.word, sc.isbwu, sc.idSuffix, null,tabs);
			
		}
	}
	
	private void moveShortBaseWordsToParentNode() throws SQLException, NumberFormatException, IOException
	{
		String sql = "Select tablename FROM map ORDER BY prefix";
		ResultSet rs = this.stat.executeQuery(sql);
		ArrayList<String> tabs = new ArrayList<String>();				
		while(rs.next())			
		{
			String table = rs.getString("tablename");	
			tabs.add(table);
		}
		
		ArrayList<StructCorrect> toRepair = new ArrayList<StructCorrect>();
		for(String table : tabs)
		{
			if(table.length() <= 2) continue;
			
			String sqlTpl = String.format("Select * FROM [%s] WHERE wrdlen = %s",table,table.length());
			rs = this.stat.executeQuery(sqlTpl);
			String newTable = table.substring(0,table.length() - 1);
			if(!tabs.contains(newTable))
			{
				handleCreateTable(newTable);
//				System.err.printf("Je potreba %s\n",newTable);
			}
			
			while(rs.next())
			{
				String word = rs.getString("wordbase");
				int idSuffix = rs.getInt("suffix");
				boolean isbwu = rs.getInt("isbwu") == 1;
				StructCorrect sc = new StructCorrect();
				sc.newTable = newTable;				
				sc.table = table;				
				sc.word = word;
				sc.idSuffix = idSuffix;
				sc.isbwu = isbwu;
				
				{
					toRepair.add(sc);
//					System.out.printf("table:%s word:%s \n",table,word);
				}
			}
			rs.close();
		}
		for(StructCorrect sc : toRepair)
		{
			String newTable = sc.table.substring(0,sc.word.length()-1);			
			sql = String.format("INSERT INTO [%s] SELECT * FROM [%s] WHERE wordbase='%s'",newTable,sc.table,sc.word);
			this.stat.execute(sql);
			
			//ze stary nemazat, protoze by se to tam zase nenaslo
			//sql = String.format("DELETE FROM [%s] WHERE wordbase='%s'",sc.table,sc.word);
			//this.stat.execute(sql);
			
		}
	}
	
	class StructCorrect
	{
		public String word;
		public String table;
		public String newTable;
		public int idSuffix;
		public boolean isbwu;
		
		@Override
		public String toString()
		{
			return String.format("%s [%s] (%s)",word,table,idSuffix);
		}
	}
	
	private boolean isCorrectLength(List<String> tables,String word)
	{
		String contName = Character.toUpperCase(word.charAt(0)) + Stringology.getGeneralizedString(word.substring(1));
		int len = contName.length();
		for(String tbl: tables)
		{
			if(tbl.startsWith(contName))
			{
				int tblLen = tbl.length();
				if(tblLen > len)
					len = tbl.length();
			}
		}
		return !(len > word.length());
	}
	
	private String getCorrectTable(List<String> tables,String word)
	{
		String normalizedPrefix = Stringology.getNormalizedString(word);
		int len = normalizedPrefix.length();
		normalizedPrefix = (Character.toUpperCase(normalizedPrefix.charAt(0)) + Stringology.getGeneralizedString(normalizedPrefix.substring(1, len)).toLowerCase());
		String substr = null;
		for (int i = len; i > 0; i--)
		{
			substr = normalizedPrefix.substring(0, i);
			if (tables.contains(substr))
			{
				return substr;				
			}
		}
		return "";
	}
	
	private void updateMapTable() throws SQLException 
	{
	
		String sql = "Select tablename FROM map ORDER BY prefix";
		String sqlct = "Select normwordbase FROM [%s] ORDER BY wordbase LIMIT 1 ";
		String sqlct2 = "Select COUNT(*) FROM [%s]";
		ResultSet rs = this.stat.executeQuery(sql);
		ArrayList<String> tabs = new ArrayList<String>();				
		while(rs.next())			
		{
			String table = rs.getString("tablename");	
			tabs.add(table);
		}
		for(String table:tabs)
		{
			//String sqlt = "UPDATE map SET fwpref = \"%s\", recnt = %s WHERE tablename = \"%s\"";
			String sqlt = "UPDATE map SET recnt = %s WHERE tablename = \"%s\"";
			String sqlc = String.format(sqlct,table);
			String sqlc2 = String.format(sqlct2,table);
				
			ResultSet rs3 = this.stat.executeQuery(sqlc2);
			int recnt = 0;
			if(rs3.next())
			{
				 recnt = rs3.getInt(1);
				 rs3.close();
			}
			ResultSet rs2 = this.stat.executeQuery(sqlc);		
			if (rs2.next())
			{
				
				String fwpref = rs2.getString("normwordbase");		
				if(fwpref.length() > 4) fwpref = fwpref.substring(0, 4);
				//String sqlup = String.format(sqlt, fwpref,recnt,table);
				String sqlup = String.format(sqlt, recnt,table);
				int q = this.stat.executeUpdate(sqlup);
				String a = String.valueOf(q);
			}
		}
	}

	private void parseFile() throws Exception
	{	
		
		this.generateDatabaseCompleteFile(params.baseWordFile, false);
		this.fillTables();
		
		this.stat.executeUpdate("DROP TABLE COMPLETE;");
		this.stat.executeUpdate("VACUUM;");
	}

	private void fillTableMap(String key) throws SQLException
	{
//		Set<String> keyss = this.tableMap.keySet();
//		List<String> keys = new ArrayList<String>(keyss);
//		Collections.sort(keys);
//		for(String key : keys)
//		{
			try
			{
			String sql = "INSERT INTO map (prefix, tablename) VALUES (\"%s\",\"%s\")";
			sql = String.format(sql,key, this.tableMap.get(key));
			stat.executeUpdate(sql);
			}
			catch(SQLException sqle)
			{
				
			}
		//}
	}
	
	private void fillTableMapComplete() throws SQLException, NumberFormatException, IOException
	{
		String sql = "SELECT tbl_name FROM sqlite_master where type ='table' And length(tbl_name) < 8 and tbl_name <> 'map' and tbl_name <> 'suff'";
		ArrayList<String> tables = new ArrayList<String>();
		ResultSet rs = this.stat.executeQuery(sql);
		
		while(rs.next())
		{
			String tbl = rs.getString("tbl_name");
			tables.add(tbl);
		}
		rs.close();
		
		for(String tbl : tables)
		{
			tableMap.put(tbl, tbl);
			fillTableMap(tbl);
		}
	}
		
	
//	HashSet<String> allTables = new HashSet<String>();
//	private void moveRecordsFromSmallTables() throws SQLException
//	{
//		String sql = "DROP TABLE IF EXISTS " + aggregTable;
//		stat.executeUpdate(sql);
//		
////		sql = "CREATE VIRTUAL TABLE \"" + aggregTable + "\" USING fts3(\"wordbase\" TEXT NOT NULL UNIQUE, \"normwordbase\" TEXT NOT NULL, \"searchtext\" TEXT NOT NULL, \"gensearchtext\" TEXT NOT NULL,  \"swype\" TEXT NOT NULL, \"genswype\" TEXT NOT NULL, \"isbwu\" INTEGER NOT NULL, \"suffix\" INTEGER NOT NULL, \"cntr\" INTEGER DEFAULT 0, \"wrdlen\" INTEGER NOT NULL, \"profil\" INTEGER )";
////		stat.executeUpdate(sql);
//		
//		sql = String.format("Select tablename, recnt FROM map where tablename<>\"%s\" ORDER BY recnt",aggregTable);
//		ResultSet rs = this.stat.executeQuery(sql);
//		List<String> tables = new ArrayList<String>();
//		while(rs.next())			
//		{
//			String table = rs.getString("tablename");
//			allTables.add(table);
//			int recCount = rs.getInt("recnt");
//			if(recCount < aggreggLimit)
//			{
//				tables.add(table);				
//			}
//		}
//		
//		for(String table : tables)
//		{
//			String it = "INSERT INTO [%s] (wordbase, normwordbase, searchtext, gensearchtext,  swype, genswype, isbwu, suffix, cntr, wrdlen) ";
//			String sqlt = it + "select wordbase, normwordbase, searchtext, gensearchtext, swype,genswype, isbwu, suffix, cntr, wrdlen " +
//			  "FROM [%s]";
//			String newTable = getNewTable(table);
//			if(!newTable.equals("") && !newTable.equals(table))
//			{
//				sqlt = String.format(sqlt, newTable ,table);
//				this.stat.execute(sqlt);
//				sqlt = String.format("DROP TABLE [%s]",table);
//				this.stat.execute(sqlt);
//				sqlt = String.format("DELETE FROM map WHERE tablename=\"%s\"",table);
//				this.stat.execute(sqlt);
//				
////				sqlt = "UPDATE map SET tablename=\"%s\" WHERE tablename=\"%s\"";
////				sqlt = String.format(sqlt,aggregTable,newTable);
////				this.stat.execute(sqlt);
//			}
//		}
//	}
	
//	private String getNewTable(String word)
//	{
//		String kWord = Character.toUpperCase(word.charAt(0)) + Stringology.getMoreGeneralizedString(word.substring(1) + "aaaaaaaaaa");
//		int len = kWord.length();
//		int limit = len - word.length();
//		String result = "";
//		for(int i = 0;i<limit;i++)
//		{
//			String subWord = Character.toUpperCase(kWord.charAt(0)) + kWord.substring(1,limit-i);
//			if(allTables.contains(subWord))
//				return subWord;
//		}
//		return result;
//	}
	
	
		
	
	private void initDbFile() throws ClassNotFoundException, SQLException {
		
		Class.forName("org.sqlite.JDBC");
	    this.conn = DriverManager.getConnection(String.format("jdbc:sqlite:%s",params.dbOutputFile));
	    this.stat = conn.createStatement();
	    this.stat.execute("PRAGMA synchronous = 0;");
	    this.stat.execute("PRAGMA cache_size = 10000;");
	    
	}

	//na tohle nezapomenout, to tam musi bejt
	private void createAndroidMetaData() throws SQLException, NumberFormatException, IOException
	{		
		String sql = "DROP TABLE IF EXISTS \"android_metadata\";";
		stat.execute(sql);
		sql = "CREATE TABLE android_metadata (locale TEXT)";
		stat.executeUpdate(sql);
		sql = "INSERT INTO android_metadata (locale) VALUES (\"en_US\")";
		stat.executeUpdate(sql);
	}
	
	@Deprecated
	private void createTables(boolean overwrite) throws SQLException, NumberFormatException, IOException
	{				
		String template = "CREATE VIRTUAL TABLE  \"%s\" USING fts3(\"wordbase\" TEXT NOT NULL UNIQUE, \"normwordbase\" TEXT NOT NULL, \"searchtext\" TEXT NOT NULL, \"gensearchtext\" TEXT NOT NULL,  \"isbwu\" INTEGER NOT NULL, \"suffix\" INTEGER NOT NULL, \"cntr\" INTEGER INTEGER DEFAULT 0, \"wrdlen\" INTEGER NOT NULL);";								
		String sql = null;
		Set<String> tables = this.tableMap.keySet();
		int cntr = 0;
		for(String key : tables)
		{
			String table = this.tableMap.get(key);
			if(overwrite)
			{
				sql = String.format("DROP TABLE IF EXISTS \"%s\"; ",table);
				this.stat.execute(sql);
			}			
			if(!this.existsTable(table))
			{
				sql = String.format(template,table);
				stat.executeUpdate(sql);		
			}
			
			if(cntr++ % 100 == 0)
			{
				System.out.println(String.format("Tbl:%s i:%s / %s",table,cntr,tables.size()));
			}
		}
	}
	
	private void createTable(String table, boolean overwrite) throws SQLException, NumberFormatException, IOException
	{				
		String template = "CREATE VIRTUAL TABLE  \"%s\" USING fts3(\"wordbase\" TEXT NOT NULL UNIQUE, \"normwordbase\" TEXT NOT NULL, \"searchtext\" TEXT NOT NULL, \"gensearchtext\" TEXT NOT NULL,  \"swype\" TEXT NOT NULL, \"genswype\" TEXT NOT NULL, \"isbwu\" INTEGER NOT NULL, \"suffix\" INTEGER NOT NULL, \"cntr\" INTEGER INTEGER DEFAULT 0, \"wrdlen\" INTEGER NOT NULL, \"profil\" INTEGER );";								
		String sql = null;		
		//for(String key : tables)
		{
			//String table = this.tableMap.get(key);
			if(overwrite)
			{
				sql = String.format("DROP TABLE IF EXISTS \"%s\"; ",table);
				this.stat.execute(sql);
			}			
			if(!this.existsTable(table))
			{
				sql = String.format(template,table);
				stat.executeUpdate(sql);		
			}
		}
	}
	
	private void createCompleteTable() throws SQLException, NumberFormatException, IOException
	{				
		String template = "CREATE VIRTUAL TABLE  \"COMPLETE\" USING fts3(\"wordbase\" TEXT NOT NULL UNIQUE, \"normwordbase\" TEXT NOT NULL, \"genwordbase\" TEXT NOT NULL,\"searchtext\" TEXT NOT NULL, \"gensearchtext\" TEXT NOT NULL, \"swype\" TEXT NOT NULL,  \"genswype\" TEXT NOT NULL, \"isbwu\" INTEGER NOT NULL, \"suffix\" INTEGER NOT NULL, \"cntr\" INTEGER INTEGER DEFAULT 0, \"wrdlen\" INTEGER NOT NULL);";								
		String sql = template;		
		stat.executeUpdate(sql);		
	}
	
	private void createSuffixTable()  throws SQLException, NumberFormatException, IOException
	{
		String template = "CREATE VIRTUAL TABLE  \"suff\" USING fts3(\"id\" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, \"suffix\" TEXT NOT NULL, \"normsuffix\" TEXT NOT NULL);";
		//String template = "CREATE VIRTUAL TABLE  \"ABCD\" USING fts3(prdel TEXT);";		
			String sql = "DROP TABLE IF EXISTS \"suff\";";
			this.stat.execute(sql);
			sql = template;
			stat.executeUpdate(sql);			
	}
	
	private void createMapTable()  throws SQLException, NumberFormatException, IOException
	{
		String template = "CREATE VIRTUAL TABLE \"map\" USING fts3(\"prefix\" TEXT NOT NULL UNIQUE, \"tablename\" TEXT NOT NULL, recnt INTEGER);";
		//String template = "CREATE VIRTUAL TABLE  \"ABCD\" USING fts3(prdel TEXT);";		
		String sql = "DROP TABLE IF EXISTS \"map\";";
		this.stat.execute(sql);
		sql = template;
		stat.executeUpdate(sql);
	}
	
	public boolean existsTable(String tablename) throws SQLException
	{
		boolean result = false;
		tablename = tablename.replace("'", "");
		String sql = "select COUNT(*) as p from sqlite_master where type = 'table' AND name='%s'";
		PreparedStatement ps = this.conn.prepareStatement(String.format(sql,tablename));
		ps.execute();
		ResultSet rs = ps.getResultSet();

		if(rs.getInt(1) == 0)
			result = false;
		else
			result = true;
		
		rs.close();
		ps.close();
		return result;		
	}
	
	//public void fillTable(char ch, String table, boolean skipFirstLine, int acceptedPrefix) throws Exception
	public void generateDatabaseFile(String file, boolean skipFirstLine) throws Exception
	{					
		if(!new File(file).exists())
			return;
		
		BufferedReader br;
		Reader in = new InputStreamReader(new FileInputStream(file),"UTF-8");
		br = new BufferedReader(in);		
				
		String line;
		int cntr = 1;
		
		if(skipFirstLine)
			br.readLine(); //headers
		while((line = br.readLine()) != null)
		{
			//if(line.startsWith("ch"))
			{
				String[] atoms = line.split(";");
				String baseword = atoms[0];			
				int basewordlen = baseword.length();
				int suffixID = 0;
				suffixID = (atoms[1].length() > 0) ? Integer.parseInt(atoms[1]) : 0;
				if(basewordlen < 4 && suffixID == 0)
					continue;
				String table = this.handleCreateTable(baseword);
				
				if(baseword.length() > lenLimit)continue;
				int usbw = Integer.parseInt(atoms[2]);
				if(suffixID == 0 && usbw == 0)
					usbw = 1;
				String normWord = Stringology.getNormalizedString(baseword);
								
				String sql = null;
				PreparedStatement ps = null;			
				
				String suff = this.normSuffixy.get(suffixID);
				String searchText = getSpaceSeparatedWord(normWord);				
				if(!(suff == null && suffixID == 0))
					searchText += " "  + this.getSearchableSuffix(suff);
				
				String genSearchText = Stringology.getGeneralizedString(searchText);				
				String genMoreSearchText = Stringology.getMoreGeneralizedString(searchText);
				sql = String.format(insertTemplate,table,baseword,normWord,searchText, genSearchText,genMoreSearchText,usbw,suffixID,basewordlen);
				ps = this.conn.prepareStatement(sql);
				ps.execute();
							
				if(++cntr % 10 == 0)
				{			
					System.out.println(String.format("RecNum:%s",cntr));			
				}
			}
		}
		br.close();
	}
	
	public void generateDatabaseCompleteFile(String file, boolean skipFirstLine) throws Exception
	{					
//		if(!new File(file).exists())
//			return;
		
		BufferedReader br;
		FileInputStream fs = new FileInputStream(file);		
		Reader in = new InputStreamReader(fs,"UTF-8");
		br = new BufferedReader(in);		
			
		String line;
		int cntr = 1;
		
		if(skipFirstLine)
			br.readLine(); //headers
		
		
		this.createCompleteTable();		
		
		while((line = br.readLine()) != null)
		{
			try
			{
//				String q = Stringology.getNormalizedString(line.substring(0,2));
//				if(line.startsWith("n"))
				{
					String[] atoms = line.split(";");
					String baseword = atoms[0];
					baseword = baseword.replace("-","");
					int basewordlen = baseword.length();
					if(basewordlen > lenLimit)continue;
					
					int suffixID = 0;
					suffixID = (atoms[1].length() > 0) ? Integer.parseInt(atoms[1]) : 0;
					if((basewordlen < 2 && suffixID == 0))
						continue;
					if(basewordlen < 4 && suffixID > 0)
					{
						boolean isbwu = Integer.parseInt(atoms[2]) == 1;
						this.handleShortWordsWithSuffis(baseword, isbwu, suffixID, "COMPLETE",null);
						continue;
					}
					
					//beru jen slova len >=4 && suffID > 0
					// nebo slova < 4 && SuffID =
					//if(!(basewordlen < 4 && suffixID > 0) || (basewordlen < 4 && suffixID > 0 && Integer.parseInt(atoms[2]) == 1))
					{
						String table = "COMPLETE";//this.getTableName(baseword);
						
						
						int usbw = Integer.parseInt(atoms[2]);
						if(suffixID == 0 && usbw == 0)
							usbw = 1;
						String normWord = Stringology.getNormalizedString(baseword);
						int angleLimit = 55;
						int angleLimit2 = 90;
						int angleLimit3 = 120;
						String ssy = this.getSwypeText(normWord,angleLimit,this.keyboard_qwerty);
						String ssyl = this.getSwypeText(normWord,angleLimit3,this.keyboard_qwerty);
						String ssyl2 = this.getSwypeText(normWord,angleLimit2,this.keyboard_qwerty);
						String ssz = this.getSwypeText(normWord,angleLimit,this.keyboard_qwertz);
						String sszl = this.getSwypeText(normWord,angleLimit3,this.keyboard_qwertz);
						String sszl2 = this.getSwypeText(normWord,angleLimit2,this.keyboard_qwertz);
						String swypeValue = this.getDistinctWords(String.format("%s %s %s %s %s %s",ssy,ssyl,ssz, sszl,ssyl2,sszl2));
						//String swypeValue = this.getDistinctWords(String.format("%s %s %s %s",ssy,ssz, ssyl2,sszl2));
	//					if(swypeValue.contains(" "))
	//						swypeValue = swypeValue.replace(normWord, "").replace("  ", " ").trim();
	//					swypeValue = getShortestWord(swypeValue);
	//					if(swypeValue.contains(" "))
	//					{
	// 						swypeValue = ""; 
	//					}
						String genSwypeValue = Stringology.getGeneralizedString(swypeValue);
						
						String sql = null;
						PreparedStatement ps = null;			
						
						String suff = this.normSuffixy.get(suffixID);
						String searchText = getSpaceSeparatedWord(normWord);				
						if(!(suff == null && suffixID == 0))
							searchText += " "  + this.getSearchableSuffix(suff);
						
//						String genNormWord = Stringology.getMoreGeneralizedString(normWord);			
						String genNormWord = Stringology.getGeneralizedString(normWord);
//						System.out.println(genNormWord);
						String genSearchText = Stringology.getGeneralizedString(searchText);	
						sql = String.format(insertCompleteTemplate,table,baseword,normWord,genNormWord, searchText, genSearchText, swypeValue,genSwypeValue,usbw,suffixID,basewordlen);
						ps = this.conn.prepareStatement(sql);
						ps.execute();
					}
								
					if(++cntr % 100 == 0)
						System.out.println(String.format("RecNum:%s",cntr));			
					if(cntr % 50000 == 0)
						resetConnection();					
				}
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		br.close();
	}
	
	public String getShortestWord(String text)
	{
		if(text.length() == 0)
			return "";
		String[] words = text.split(" ");
		String result = words[0];
		for(int i = 1;i<words.length;i++)
		{
			String newOne = words[i];
			if(newOne.length() < result.length())
				result = newOne;
		}
		return result;
		
	}
	
	/**
	 * Protection for OutOfMemoryException
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 */
	private void resetConnection() throws SQLException, ClassNotFoundException
	{
		this.stat.close();
		this.conn.close();
		this.initDbFile();
		//System.out.println("REOPEN DB");
	}
	
	
	
	private void handleShortWordsWithSuffis(String baseWord, boolean isbwu, int idSuffix, String toTable, List<String> tables) throws SQLException
	{
		
		String[] arrr = this.suffixy.get(idSuffix).split(" ");
		List<String> sufy = new ArrayList<String>();
		for(String s:arrr)
			sufy.add(s);
		//this.getTableName(baseword);
		int suffixID = 0;
		if(isbwu)
			sufy.add(0, "");
		
		int usbw = 1;		
		for(String suf : sufy)
		{
			String baseword = baseWord + suf;
			String normWord = Stringology.getNormalizedString(baseword);
			int angleLimit = 55;
			int angleLimit2 = 90;
			String table = toTable;
			if(table == null)
				  table = getCorrectTable(tables, baseword);
			String ssy = this.getSwypeText(normWord,angleLimit,this.keyboard_qwerty);
			String ssyl = this.getSwypeText(normWord,120,this.keyboard_qwerty);
			String ssyl2 = this.getSwypeText(normWord,angleLimit2,this.keyboard_qwerty);
			String ssz = this.getSwypeText(normWord,angleLimit,this.keyboard_qwertz);
			String sszl = this.getSwypeText(normWord,120,this.keyboard_qwertz);
			String sszl2 = this.getSwypeText(normWord,angleLimit2,this.keyboard_qwertz);
			String swypeValue = this.getDistinctWords(String.format("%s %s %s %s %s %s",ssy,ssyl,ssz, sszl,ssyl2,sszl2));
			//String swypeValue = this.getDistinctWords(String.format("%s %s %s %s",ssy,ssz, ssyl2,sszl2));
			String genSwypeValue = Stringology.getGeneralizedString(swypeValue);
			String sql = null;
		
			String searchText = getSpaceSeparatedWord(normWord);	
			String genNormWord = Stringology.getGeneralizedString(normWord);			
			String genSearchText = Stringology.getGeneralizedString(searchText);				
//			String genMoreSearchText = Stringology.getMoreGeneralizedString(searchText);		
			if(table.equals("COMPLETE"))
				sql = String.format(insertCompleteTemplate,table,baseword,normWord,genNormWord, searchText, genSearchText, swypeValue,genSwypeValue,usbw,suffixID,baseword.length());
			else
				sql = String.format(insertTemplate,table,baseword,normWord, searchText, genSearchText, swypeValue,genSwypeValue,usbw,suffixID,baseword.length());
			PreparedStatement ps = null;		
			ps = this.conn.prepareStatement(sql);
			ps.execute();
		}
		
	}
	private String getDistinctWords(String text)
	{
		HashSet<String> hs = new HashSet<String>();
		String[] words = text.split(" ");
		StringBuilder sb = new StringBuilder();
		for(String w:words)
		{
			if(!hs.contains(w))
			{
				sb.append(w + " "); 
				hs.add(w);
			}
		}
		return sb.substring(0,sb.length() -1);
	}
	private String getSpaceSeparatedWord(String word)
	{
		StringBuilder sb = new StringBuilder();
		for(char c : word.toCharArray())
		{
			sb.append(c + " ");
		}
		return sb.substring(0, sb.length() - 1);
	}
	
	private void fillTables() throws SQLException, NumberFormatException, IOException, ClassNotFoundException
	{
		String abeceda = "abcdefghijklmnopqrstuvwxyz".toUpperCase();
		//abeceda = "n".toUpperCase();
		char[] chars = abeceda.toCharArray();
		boolean cont = false;
		for(int i = 0;i<chars.length;i++)
		{			
			String c = String.valueOf(chars[i]);
			String sql = String.format("Select COUNT(*) as pocet FROM COMPLETE WHERE normwordbase MATCH '%s*'",c);
			ResultSet rs = this.stat.executeQuery(sql);
			int pocet = 0;
			if(rs.next())
			{
				pocet = rs.getInt(1);
			}
			rs.close();
			
			if(pocet > maxRecordsInTable)
			{
				fillTablesSub(c.toLowerCase(), null, 2);
			}
			else
			{
				String tbl = c;
				String sqlq = null;
				
				//if(whereGenWordCondition == null)
				{	
					
					this.stat.executeUpdate("DROP TABLE IF EXISTS " + tbl);
					String whereToSQL = "normwordbase LIKE '" + c + "%'";
					String it = "INSERT INTO [%s] (wordbase, normwordbase, searchtext, gensearchtext,  swype, genswype, isbwu, suffix, cntr, wrdlen) ";
					String sqlt = it + "select wordbase, normwordbase, searchtext, gensearchtext, swype,genswype, isbwu, suffix, cntr, wrdlen " +
					  "FROM COMPLETE WHERE %s ";
					this.handleCreateTable(tbl);
					sqlq = String.format(sqlt,tbl,whereToSQL);
				}				
				System.out.println(tbl);
				this.resetConnection();
				this.stat.executeUpdate(sqlq);
			}
			//this.resetConnection();
		}
	}
	
	public void fillTablesSub(String whereNormWordCondition, String whereGenWordCondition, int level) throws SQLException, NumberFormatException, IOException, ClassNotFoundException
	{
		
		String whereToSQL = whereNormWordCondition;
		String whereToSQL2 = whereGenWordCondition;
		if(!whereToSQL.contains(" ")) 
			whereToSQL = "normwordbase LIKE '" + whereToSQL + "%'";
		
		if(whereGenWordCondition != null) 
			whereToSQL2 = "genwordbase LIKE '" + whereToSQL2 + "%'";
		
		String sql = null;
		if(whereGenWordCondition == null)
			sql = String.format("Select COUNT(*) as pocet, substr(genwordbase,1,%s) as iq  FROM COMPLETE WHERE %s GROUP BY substr(genwordbase,1,%s)",level,whereToSQL,level);
		else
			sql = String.format("Select COUNT(*) as pocet, substr(genwordbase,1,%s) as iq  FROM COMPLETE WHERE %s AND %s  GROUP BY substr(genwordbase,1,%s)",level,whereToSQL,whereToSQL2,level);
		ResultSet rs = this.stat.executeQuery(sql);
		int pocet = 0;
		String np = null;
		HashMap<String, Integer> gm = new HashMap<String, Integer>();
		HashMap<String, Integer> mgm = new HashMap<String, Integer>();
		
		
		while(rs.next())
		{
			pocet = rs.getInt(1);
			np = rs.getString(2);
			//System.out.println(String.format("CHAR:%s Count:%S",np,pocet));
			//String mgenVal = whereNormWordCondition + KeyboardtUtils.getMoreGeneralizedString(np.substring(level-1));
			String mgenVal = np;	
			if(isCorrectWord(mgenVal))
			{
				if(mgm.containsKey(mgenVal))
					mgm.put(mgenVal, mgm.get(mgenVal) + pocet);
				else
					mgm.put(mgenVal, pocet);
			}
			else
			{
				System.err.printf("Incorrect word:'%s' for prefix! (SKIPPING)\n", mgenVal);
			}
		}
		rs.close();
		
		for(String s : mgm.keySet())
		{
			int pct = mgm.get(s);
			if(pct > maxRecordsInTable)
			{
				fillTablesSub(whereNormWordCondition, s,level + 1);
			}
			else
			{
				
				String it = "INSERT INTO [%s] (wordbase, normwordbase, searchtext, gensearchtext,  swype, genswype, isbwu, suffix, cntr, wrdlen) ";
				String tbl = whereNormWordCondition.toUpperCase();
				this.stat.executeUpdate("DROP TABLE IF EXISTS " + tbl);
				String sqlq = null;
				if(s.length() < level)
				{
					{
						String sqlt = it + "select wordbase, normwordbase, searchtext, gensearchtext,  swype, genswype, isbwu, suffix, cntr, wrdlen " +
						  "FROM COMPLETE WHERE %s AND genwordbase LIKE '%s%%' AND LENGTH(wordbase) = " + s.length();
						tbl = tbl + s.substring(1);
						sqlq = String.format(sqlt,tbl,whereToSQL,s);
					}
				}
				else
				{
					{
						String sqlt = it + "select wordbase, normwordbase, searchtext, gensearchtext,  swype, genswype, isbwu, suffix, cntr, wrdlen " +
						  "FROM COMPLETE WHERE %s AND genwordbase LIKE '%s%%'";
						tbl = tbl + s.substring(1);
						sqlq = String.format(sqlt,tbl,whereToSQL,s);
					}
				}
				System.out.println(tbl + " " + pct);
				this.resetConnection();
				this.handleCreateTable(tbl);
				this.stat.executeUpdate(sqlq);
			}
		}
	}
	
	private boolean isCorrectWord(String text)
	{
		for(char c :text.toCharArray())
		{
			if(!Character.isLetter(c))
				return false;
		}
		return true;
	}
	
	public void fillSuffixTable() throws NumberFormatException, IOException, SQLException
	{
		//String file = String.format(this.filesLocation,"suffy");
		BufferedReader br;
		Reader in = new InputStreamReader(new FileInputStream(params.suffixFile),"UTF-8");
		br = new BufferedReader(in);		
		final String insertTemplate = "INSERT INTO \"suff\" (\"id\",\"suffix\", \"normsuffix\") VALUES (%s,\"%s\",\"%s\");";
		
		String line;
		int cntr = 1;		
		
		while((line = br.readLine()) != null)
		{
			String[] atoms = line.split(";");
			int id = Integer.parseInt(atoms[0]);
			String suffix = atoms[1].replace(',', ' ');
			String normSuffix = Stringology.getNormalizedString(suffix);
			String sql = String.format(insertTemplate,id,suffix,normSuffix);
			PreparedStatement ps = this.conn.prepareStatement(sql);
			ps.execute();
			//sb.append(sql);
			this.normSuffixy.put(id, normSuffix);
			this.suffixy.put(id, suffix);
			if(++cntr % 10 == 0)				
				System.out.println(String.format("Processed suffixfile, RecNum:%s",cntr));						
		}
		br.close();
	}
	
	public String getTableName1(String word)
	{
		int len = word.length();
		if(len > 3) len = 3;	
		String substr = "";
		String tableName = "";
		for(int i = 0;i<len;i++)
		{
			substr = substr + Stringology.getNormalizedString(word.substring(i,i+1));			
			if(this.tableMap.containsKey(substr))
				tableName = this.tableMap.get(substr);			
		}
		return tableName;
	}
	
	public String handleCreateTable(String tbl) throws NumberFormatException, SQLException, IOException
	{
		try
		{
			
//			switch(len)
//			{		
//				case 2:
//					char c1 = KeyboardtUtils.getMoreGeneralizedChar(word.charAt(1));
//					tbl = tbl + c1;
//				break;
//				case 3:					
//					tbl = tbl + genWord.charAt(1) + genWord.charAt(2) + genWord.charAt(2);
//					break;
//				default:
//					//tbl = tbl + word.charAt(0) + KeyboardtUtils.getMoreGeneralizedChar(word.charAt(1)) + word.charAt(2);
//					tbl = tbl + word.charAt(0) + genWord.charAt(1) + genWord.charAt(2) + genWord.charAt(3);
//				break;
//			}
			if(!this.tableMap.containsKey(tbl))
			{
				this.createTable(tbl, true);
				this.tableMap.put(tbl, tbl);
				this.fillTableMap(tbl);
			}
		}
		catch(StringIndexOutOfBoundsException se)
		{
			se.printStackTrace();
		}
		return tbl;
	}
	

	
	public String getDistinctString(String source)
	{
		HashSet<Character> chars = new HashSet<Character>();
		StringBuilder sb = new StringBuilder();
		for(char c : source.toCharArray())
		{
			if(!chars.contains(c))
			{
				sb.append(c);
				chars.add(c);
			}
		}
		return sb.toString();
	}
	

	
	
	
	private void insertIntoTableMap(String tableName,String[] prefixes)
	{
		for(String key:prefixes)
			this.tableMap.put(key,tableName);
	}
	
	private void loadTableMap() throws SQLException
	{
				
		String sql = "select * FROM map";
		PreparedStatement ps = this.conn.prepareStatement(sql);
		ps.execute();
		ResultSet rs = ps.getResultSet();
		while(rs.next())
		{
			String tbl = rs.getString("tablename");
			this.tableMap.put(tbl, tbl);
		}
		
		rs.close();
		ps.close();
	}
	
	private void echoRecordCount() throws SQLException
	{
		String sqlt = "Select Count(*) as x FROM [%s]";
		Set<String> keyss = this.tableMap.keySet();
		List<String> keys = new ArrayList<String>(keyss);
		Collections.sort(keys);
		List<String> tables = new ArrayList<String>();
		HashMap<Integer, String> q = new HashMap<Integer, String>();
				
		
		for(String key : keys)
		{
			String table = this.tableMap.get(key);											
			if(!tables.contains(table))
				tables.add(table);
		}
				
		for(String table:tables)
		{			
			String sql = String.format(sqlt,table);
			ResultSet rs = this.stat.executeQuery(sql);
			int pocet = 0;
			while(rs.next())			
				pocet = rs.getInt(1);						
			System.out.println(String.format("Table: %s  RecordCount: %s",table,pocet));
		}
	}		
	
	private String getSearchableSuffix(String suffix)
	{
		StringBuilder sb = new StringBuilder();
		for(char c : suffix.toCharArray())
		{
			if(sb.indexOf(String.valueOf(c)) == -1)
				sb.append(c + " ");
		}
		return sb.substring(0,sb.length() - 1);
	}
	
	public String getSwypeText(String normalizedWord, int angleLimit, Keyboard keyboard)
	{
		String test = Stringology.removeDuplicateCharacters(normalizedWord.toLowerCase());
		ArrayList<PointF> l = keyboard.getPoints(test);
		ArrayList<PointF> simple = ExtMath.getSimpleMutliLine(l,angleLimit,test);
		
		StringBuilder sb = new StringBuilder();
		for(PointF p:simple)
		{
			Key k = keyboard.getKey(p.x, p.y);
			sb.append(k.c);
		}
		int pocet = simple.size();
		//System.out.println("Short:" + sb.toString() + " pocet:" +pocet);
		return sb.toString();
		
	}
	
	
	public Keyboard loadKeyboardXML(String sfile)	
	{
		
		Keyboard keyboard = new Keyboard();
		int completeWidth = 320;
		float defaultWidth = 0.1f;
		try {
			//File file = new File(sfile);
			String file = String.format("/com/scurab/dictgen/%s",sfile) ;
			  //URL url = getClass().getSystemResource(file);
		   InputStream isr = getClass().getResourceAsStream(file);
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document doc = db.parse(isr);
			doc.getDocumentElement().normalize();
			NodeList nl = doc.getElementsByTagName("Row");
			
			int cnt = nl.getLength();
			int top = 0;
			for(int i = 0;i<cnt;i++) 
			{
				Node n = nl.item(i);
				if(n.getNodeType() == Node.ELEMENT_NODE)
				{
					Element el = (Element)n;
					NodeList keys = el.getElementsByTagName("Key");
					int cntkeys = keys.getLength();

					Row r = keyboard.new Row();
					r.height = 50;
					int left = 0;
					for(int j = 0;j<cntkeys;j++)
					{
						Node chn = keys.item(j);
						if(chn.getNodeType() == Node.ELEMENT_NODE)
						{
							Element key = (Element)chn;
							String codes = key.getAttribute("android:codes");							
							String[] aCodes = codes.split(",");
							
							int code = Integer.parseInt(aCodes[0]);
							char c = (char)code;
							//System.out.println(c);
							Key k = keyboard.new Key();
							k.c =c ;
							k.code = c;
							if(key.hasAttribute("android:keyWidth"))
							{
								String prc = key.getAttribute("android:keyWidth");
								prc = prc.replace("%p", "");prc = prc.replace("%", "");
								float f = Float.parseFloat(prc) / 100;
								k.width = completeWidth * f;
							}
							else
								k.width = completeWidth * defaultWidth;
							k.left = left;
							r.keys.add(k);
							left += k.width;
							
						}
					}
					r.top = top;
					top += r.height + 1;					
					keyboard.rows.add(r);
				}
			}
			

		} catch (Exception e) {
			e.printStackTrace();
		}
		return keyboard;
	}
}
