package com.scurab.dictgen;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class DBGeneratorOld {

	final String insertTemplate = "INSERT INTO \"%s\" (\"wordbase\",\"srch\",\"scnd\",\"suffix\",\"cntr\") VALUES (\"%s\",\"%s\",%s,%s,0);";
	
	private static int ACCEPT_PREFIX_ALL = 0;
	private static int ACCEPT_PREFIX_N_AND_NOT_NE_AND_NOT_NEJ = 1;
	private static int ACCEPT_PREFIX_NE_AND_NOT_NEJ = 2;
	private static int ACCEPT_PREFIX_NEJ = 3;
	
	private String filesLocation = null;
	private String dbFile = null;
	
	private int lenLimit = 10;
	
	Connection conn = null;
    Statement stat = null;
    
    
	public DBGeneratorOld(String dbFile) {
		this.dbFile = dbFile;
	}
	
	public void start(String filesLocation) throws Exception
	{		
		this.filesLocation = filesLocation + "%s.dict";
		this.initDbFile();
		this.createAndroidMetaData();
		this.createTableForSuffixes();
		this.fillSuffixTable();
		//this.fillSuffixTable2();
		this.parseFiles();
		
	}
	
	private void parseFiles() throws Exception
	{		
		
		//String template = "CREATE VIRTUAL TABLE  \"ABCD\" USING fts3(prdel TEXT);";
		String abeceda = "aábcčdďeěéfghiíjklmnoópqrřsštťuúůvwxyzž";
		
		char[] abcd = abeceda.toCharArray();
		for(int i = 0;i<abcd.length;i++)
		{									
			char ch = abcd[i];
			char normalizedChar = Stringology.getNormalizedChar(ch);
			this.processTable(ch, normalizedChar);
			
		}		
			
	}
	
	private void processTable(char ch, char normalizedChar) throws Exception
	{
		if(normalizedChar == 'n')
			this.processTableN(ch, normalizedChar);
		else
			this.processTableNotN(ch, normalizedChar);
	}
	
	private void processTableN(char ch, char normalizedChar) throws Exception
	{
		String letter = String.valueOf(normalizedChar);
		String tableN = "C" + (int)normalizedChar;
		this.createTable(tableN,false);
		this.fillTable(String.valueOf(ch),tableN,false,DBGeneratorOld.ACCEPT_PREFIX_N_AND_NOT_NE_AND_NOT_NEJ);
		this.createTable("CNe",false);
		this.fillTable("ne","CNe",false,DBGeneratorOld.ACCEPT_PREFIX_NE_AND_NOT_NEJ);
		this.createTable("CNej",false);
		this.fillTable("nej","CNej",false,DBGeneratorOld.ACCEPT_PREFIX_NEJ);
	}
	
	private void processTableNotN(char ch, char normalizedChar) throws Exception
	{
		String letter = String.valueOf((int)normalizedChar);
		String table = this.getTableName(String.valueOf(normalizedChar));
		this.createTable(table,false);		
		this.fillTable(String.valueOf(ch),table,false,DBGeneratorOld.ACCEPT_PREFIX_ALL);
	}
	
	private void initDbFile() throws ClassNotFoundException, SQLException {
		Class.forName("org.sqlite.JDBC");
	    this.conn = DriverManager.getConnection(String.format("jdbc:sqlite:%s",this.dbFile));
	    this.stat = conn.createStatement();
	    this.stat.execute("PRAGMA synchronous = 0;");
	}

	private void createAndroidMetaData() throws SQLException, NumberFormatException, IOException
	{		
		String sql = "DROP TABLE IF EXISTS \"android_metadata\";";
		stat.execute(sql);
		sql = "CREATE TABLE android_metadata (locale TEXT)";
		stat.executeUpdate(sql);
		sql = "INSERT INTO android_metadata (locale) VALUES (\"en_US\")";
		stat.executeUpdate(sql);
	}
	
	private void createTable(String table, boolean overwrite) throws SQLException, NumberFormatException, IOException
	{				
		String template = "CREATE VIRTUAL TABLE  \"%s\" USING fts3(\"wordbase\" TEXT NOT NULL, \"srch\"  TEXT NOT NULL, \"scnd\" INTEGER NOT NULL,  \"suffix\" INTEGER NOT NULL, \"cntr\" INTEGER INTEGER DEFAULT 0);";								
		String sql = null;
		if(overwrite)
		{
			sql = String.format("DROP TABLE IF EXISTS \"%s\"; ",table);
			this.stat.execute(sql);
		}
		else
		{
			if(!this.existsTable(table))
			{
				sql = String.format(template,table);
				stat.executeUpdate(sql);		
			}
		}			
	}
	
	private void createTableForSuffixes()  throws SQLException, NumberFormatException, IOException
	{
		String template = "CREATE TABLE  \"suff\" (\"id\" INTEGER PRIMARY KEY  NOT NULL  UNIQUE, \"suffix\" TEXT);";
		//String template = "CREATE VIRTUAL TABLE  \"ABCD\" USING fts3(prdel TEXT);";		
			String sql = "DROP TABLE IF EXISTS \"suff\";";
			this.stat.execute(sql);
			sql = template;
			stat.executeUpdate(sql);
				
	}
	
	public boolean existsTable(String tablename) throws SQLException
	{
		boolean result = false;
		
		String sql = "select COUNT(*) as p from sqlite_master where type = 'table' AND name='%s'";
		PreparedStatement ps = this.conn.prepareStatement(String.format(sql,tablename));
		ps.execute();
		ResultSet rs = ps.getResultSet();

		if(rs.getInt(1) == 0)
			result = false;
		else
			result = true;
		
		rs.close();
		ps.close();
		return result;
		
	}
	
	private String getTableName(String baseword) throws Exception
	{
		String result = "C%s";		
		if(baseword.startsWith("ne") && !baseword.startsWith("nej"))
			result = String.format(result, "Ne");		
		else if(baseword.startsWith("nej"))
			result = String.format(result, "Nej");		
		else
			result = String.format(result, (int)baseword.charAt(0));		
		return result;
	}
	
	
	
	//public void fillTable(char ch, String table, boolean skipFirstLine, int acceptedPrefix) throws Exception
	public void fillTable(String file, String table, boolean skipFirstLine, int acceptedPrefix) throws Exception
	{					
		skipFirstLine = true;
		char ch = '0';
		if(file.length() == 1)
		{
			ch = file.charAt(0);
			file = String.format(this.filesLocation,(int)ch);
		}
		else
			file = String.format(this.filesLocation,file);
		
		if(!new File(file).exists())
			return;
		
		BufferedReader br;
		Reader in = new InputStreamReader(new FileInputStream(file),"UTF-8");
		br = new BufferedReader(in);		
				
		String line;
		int cntr = 1;
		
		if(skipFirstLine)
			br.readLine(); //headers
		
		while((line = br.readLine()) != null)
		{
			String[] atoms = line.split(";");
			String baseword = atoms[0];			
			
			if(baseword.length() > lenLimit)continue;
			
			int charcode = '0';
			String lower = baseword.toLowerCase();
			
			if((acceptedPrefix == DBGeneratorOld.ACCEPT_PREFIX_ALL)) 
			{
				charcode = (int)baseword.charAt(1);
				if(baseword.length() > lenLimit)continue;
			}
			else if (acceptedPrefix == DBGeneratorOld.ACCEPT_PREFIX_N_AND_NOT_NE_AND_NOT_NEJ && !lower.startsWith("ne"))
			{
				charcode = (int)baseword.charAt(1);
				if(baseword.length() > lenLimit)continue;
			}	
			else if (acceptedPrefix == DBGeneratorOld.ACCEPT_PREFIX_NEJ && lower.startsWith("nej"))
			{
				charcode = (int)baseword.charAt(3);
				if(baseword.length() + 3 > 17)continue;
			}
			else if (acceptedPrefix == DBGeneratorOld.ACCEPT_PREFIX_NE_AND_NOT_NEJ && (lower.startsWith("ne") && !lower.startsWith("nej")))
			{
				if(baseword.length() + 2 > lenLimit)continue;
				charcode = (int)baseword.charAt(2);
			}
			
			
			int suffixID = Integer.parseInt(atoms[2]);
			String normWord = Stringology.getGeneralizedString(baseword);
			String sql = null;
			PreparedStatement ps = null;			
			sql = String.format(insertTemplate,table,baseword,normWord,charcode,suffixID);
			
			
			if((acceptedPrefix == DBGeneratorOld.ACCEPT_PREFIX_ALL) ||
					(acceptedPrefix == DBGeneratorOld.ACCEPT_PREFIX_N_AND_NOT_NE_AND_NOT_NEJ && !lower.startsWith("ne")) ||
						(acceptedPrefix == DBGeneratorOld.ACCEPT_PREFIX_NEJ && lower.startsWith("nej")) ||
							(acceptedPrefix == DBGeneratorOld.ACCEPT_PREFIX_NE_AND_NOT_NEJ && (lower.startsWith("ne") && !lower.startsWith("nej"))))
			{
				ps = this.conn.prepareStatement(sql);
				ps.execute();
			}
			
			
			if(++cntr % 10 == 0)
			{			
				System.out.println(String.format("Processed file:%s.dict, RecNum:%s",ch,cntr));			
			}
		}
		br.close();
	}
	
	public void fillSuffixTable() throws NumberFormatException, IOException, SQLException
	{
		String file = String.format(this.filesLocation,"suffixes");
		BufferedReader br;
		Reader in = new InputStreamReader(new FileInputStream(file),"UTF-8");
		br = new BufferedReader(in);		
		final String insertTemplate = "INSERT INTO \"suff\" (\"id\",\"suffix\") VALUES (%s,\"%s\");";
		
		String line;
		int cntr = 1;		
		br.readLine();//0 neni potreba		
		while((line = br.readLine()) != null)
		{
			line = line.replace(',', ';');
			
			int id = cntr;			
			
			String sql = String.format(insertTemplate,id,line);
			PreparedStatement ps = this.conn.prepareStatement(sql);
			ps.execute();
			//sb.append(sql);
			if(++cntr % 10 == 0)				
				System.out.println(String.format("Processed suffixfile, RecNum:%s",cntr));						
		}
		br.close();
	}
	
	public void fillSuffixTable2() throws NumberFormatException, IOException, SQLException
	{
		String file = String.format(this.filesLocation,"suffixes");
		BufferedReader br;
		Reader in = new InputStreamReader(new FileInputStream(file),"UTF-8");
		br = new BufferedReader(in);		
		final String insertTemplate = "INSERT INTO \"suff\" (\"id\",\"suffix\") VALUES (%s,\"%s\");";
		
		String line;
		int cntr = 1;		
		br.readLine();//0 neni potreba		
		while((line = br.readLine()) != null)
		{
			String[] atoms = line.split(",");
			
			int id = Integer.parseInt(atoms[0]);
			String suffix = atoms[1];
			
			String sql = String.format(insertTemplate,id,suffix);
			PreparedStatement ps = this.conn.prepareStatement(sql);
			ps.execute();
			//sb.append(sql);
			if(++cntr % 10 == 0)				
				System.out.println(String.format("Processed suffixfile, RecNum:%s",cntr));						
		}
		br.close();
	}
}
