package com.scurab.dictgen;

import java.util.ArrayList;

import com.scurab.dictgen.Keyboard.Key;

public class ExtMath
{
	/**
	 * Returns length between two points
	 * @param from
	 * @param to
	 * @return
	 */
	
	public static double getLength(PointF from, PointF to)
	{
		
		double m = to.x - from.x;
		double n = to.y - from.y;
		m = Math.pow(m, 2);
		n = Math.pow(n, 2);
		return Math.sqrt(m + n);		
	}	
	
	/**
	 * Returns angle of vector against axis x, 
	 * but result is in {0,180}, bigger angle is negativ value. (350 = -10)
	 * @param from
	 * @param to
	 * @return
	 */
	
	public static double getAngleSpecial(PointF from, PointF to)
	{		
		double v1 = to.x - from.x;
		double v2 = to.y - from.y;
		double u1 = 1;  //A[1,0]
		double u2 = 0;
		
		double cosFi = (u1*v1 + u2*v2) / (Math.sqrt(Math.pow(u1,2) + Math.pow(u2,2)) * Math.sqrt(Math.pow(v1,2) + Math.pow(v2,2)));
		double angle = Math.acos(cosFi) * 180 / Math.PI; //radians
		int subResult = (int)(angle * 1000);
		angle = subResult / 1000;
		
		if ((v1 > 0 && v2 > 0) || (v1 < 0 && v2 > 0))
			angle = - angle;
		
		if(angle == 360)
			angle = 0;
		return angle;		
	}
	
	/**
	 * Returns angle of vector against axis x
	 * @param from
	 * @param to
	 * @return
	 */
	
	public static double getAngle(PointF from, PointF to)
	{		
		double v1 = to.x - from.x;
		double v2 = to.y - from.y;
		double u1 = 1;  //A[1,0]
		double u2 = 0;
		
		double cosFi = (u1*v1 + u2*v2) / (Math.sqrt(Math.pow(u1,2) + Math.pow(u2,2)) * Math.sqrt(Math.pow(v1,2) + Math.pow(v2,2)));
		double angle = Math.acos(cosFi) * 180 / Math.PI; //radians
		int subResult = (int)(angle * 1000);
		angle = subResult / 1000;
		
		if ((v1 > 0 && v2 > 0) || (v1 < 0 && v2 > 0))
			angle = 360 - angle;
		
		if(angle == 360)
			angle = 0;
		return angle;		
	}
	
	public static ArrayList<PointF> getSimpleMutliLine(ArrayList<PointF> points, int angleLimit,String text)
	{
		int cnt = points.size();
		PointF minuly = points.get(0);		
		Smer s = Smer.START;
		ArrayList<PointF> result = new ArrayList<PointF>();
		Smer s1= Smer.START;
		Smer s2= Smer.START;
		result.add(points.get(0));
		
		for(int i = 1;i<cnt -1;i++)
		{
			PointF nyni = points.get(i);
//			if(keyboard != null)
//			{
//				System.out.println(keyboard.getKey(nyni.x, nyni.y).c);
//			}
			PointF dalsi = points.get(i+1);
			double uhelDoNyni = getAngleSpecial(minuly, nyni);
			s1 = getSmer(uhelDoNyni);
						
			double uhelDoDalsi = getAngleSpecial(nyni,dalsi);
			double diff = Math.abs(uhelDoDalsi - uhelDoNyni);
			if(diff  > 180)
				diff = 360 - diff;
			
			if(diff > angleLimit)
			{
				s2 = getSmer(uhelDoDalsi);
				if(s2 != s1)
					result.add(nyni);;
			}
			
			
			
//			System.out.println("S1:" + s1.toString() + " S2:" + s2.toString() + " (" + text.charAt(i) + "->" + text.charAt(i+1) + ") Uhel:" + diff );
			minuly = nyni;
		}
		int top = points.size() - 1;
		PointF nyni = points.get(top);
		double uhelDoNyni = getAngleSpecial(minuly, nyni);
		s1 = getSmer(uhelDoNyni);
		//	result.remove(result.size() - 1);
		
		result.add(nyni);
		return result;
	}
	
	public static Smer getSmer(double uhel)
	{
		Smer r = Smer.START;
		switch((int)uhel)
		{
			case 0:
				r = Smer.V;
				break;
			case 90:
				r = Smer.S;
				break;
			case 180:
				r = Smer.Z;
				break;
			case -90:
				r = Smer.J;
				break;
		}
		if(uhel > 0 && uhel < 90)
			r = Smer.SV;
		if(uhel > 90 && uhel < 180)
			r = Smer.SZ;
		
		if(-uhel > 0 && -uhel < 90)
			r = Smer.JV;
		if(-uhel > 90 && -uhel < 180)
			r = Smer.JZ;
		
		return r;		
	}
	
	public enum Smer
	{
		S, SV, V, JV, J, JZ, Z, SZ, START
	}
}